package com.usky.dxtop.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.usky.dxtop.common.core.page.CommonPage;
import com.usky.dxtop.common.exception.CustomException;
import com.usky.dxtop.common.utils.Arith;
import com.usky.dxtop.common.utils.QRCodeUtil;
import com.usky.dxtop.common.utils.StringUtils;
import com.usky.dxtop.common.utils.http.HttpUtils;
import com.usky.dxtop.common.utils.spring.GlobalUtils;
import com.usky.dxtop.mapper.OrderMapper;
import com.usky.dxtop.model.CallApiLog;
import com.usky.dxtop.model.Order;
import com.usky.dxtop.model.TopRadio;
import com.usky.dxtop.service.CallApiLogService;
import com.usky.dxtop.service.OrderService;
import com.usky.dxtop.service.api.TopApi;
import com.usky.dxtop.service.emun.OrderPayType;
import com.usky.dxtop.service.emun.OrderStatus;
import com.usky.dxtop.service.vo.OrderRequest;
import org.apache.logging.log4j.core.util.JsonUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.TreeMap;

/**
 * <p>
 * 订单表 服务实现类
 * </p>
 *
 * @author yq
 * @since 2021-08-26
 */
@Service
public class OrderServiceImpl extends ServiceImpl<OrderMapper, Order> implements OrderService {

    public static final String SCAN_PAY = "扫码支付";

    public static final String FIND_ORDER_INFO = "交易查询";

    public static final String CARD_TOP = "卡充值";

    @Autowired
    private CallApiLogService callApiLogService;

    @Override
    public boolean add(Order order) {
        order.setOrderFlag(OrderStatus.SUCCESS.getCode());
        //调用卡充值api
        return this.save(order);
    }

    @Override
    public CommonPage<Order> list(OrderRequest orderRequest) {
        IPage<Order> page = new Page<>(orderRequest.getCurrent(), orderRequest.getSize());
        LambdaQueryWrapper<Order> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper
                .like(StringUtils.isNotBlank(orderRequest.getUserName()),Order::getUserName,orderRequest.getUserName())
                .like(StringUtils.isNotBlank(orderRequest.getUserPhone()),Order::getUserPhone,orderRequest.getUserPhone())
                .eq(null != orderRequest.getPayType(),Order::getPayType,orderRequest.getPayType())
                .between(null != orderRequest.getStartMoney() && null != orderRequest.getEndMoney(),
                        Order::getMoney,orderRequest.getStartMoney(),orderRequest.getEndMoney())
                .between(null != orderRequest.getStartTime() && null != orderRequest.getEndTime(),
                        Order::getCreateTime,orderRequest.getStartMoney(),orderRequest.getEndTime())
                .eq(null != orderRequest.getOrderFlag(),Order::getOrderFlag,orderRequest.getOrderFlag());
        page = this.page(page, queryWrapper);
        return new CommonPage<>(page.getRecords(),page.getTotal(),page.getCurrent(),page.getSize());
    }


    /**
     * 东信扫码支付api
     * @param order
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public String getTopScanPayApi(Order order){
        String orderNumber = TopApi.getOrderNumber();
        order.setOrderNumber(orderNumber);
        TreeMap<String, String> params = TopApi.generateScanPayApiParam(order.getOrderNumber(),
                                        new Double(Arith.mul(order.getMoney(),100)).intValue(), null,
                null,null,null,null,null,null);
        String scanPayUrl = String.format("%s?%s&sign=%s", TopApi.SCAN_TO_PAY_URL,
                TopApi.joinParam(params),
                TopApi.signData(params));
        baseMapper.insert(order);
        //记录调用日志
        callApiLogService.add(orderNumber,SCAN_PAY,TopApi.SCAN_TO_PAY_URL,JSONObject.toJSONString(params),null);
        return scanPayUrl;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public String getTopScanPayQr(Order order) {
        String destPath;
        try {
            //获取url
            String topScanPayApi = getTopScanPayApi(order);
            String codeName = String.format("%s-%s.jpg", "usky", System.currentTimeMillis() + "");
            destPath = String.format("%s%s", GlobalUtils.getTempBaseDir(),codeName);
            //生成二维码
            QRCodeUtil.encode(topScanPayApi, null, destPath, true);
        }catch (Exception e){
            throw new CustomException("生成二维码失败");
        }
        return destPath;
    }

    @Override
    public String callTopFindOrderInfoApi(String orderNumber) {
        TreeMap<String, String> param = TopApi.generateDealSelectParam(orderNumber, null, null, null);
        String format = String.format("%s&sign=%s", TopApi.joinParam(param),
                TopApi.signData(param));
        String result = HttpUtils.sendGet(TopApi.DEAL_SELECT_URL, format);
        JSONObject obj= JSON.parseObject(result);
//        obj.get("status")
        callApiLogService.add(orderNumber,FIND_ORDER_INFO,TopApi.DEAL_SELECT_URL,JSONObject.toJSONString(param),result);
        //成功调用卡充值api,失败记录状态
        return obj.toJSONString();
    }

    @Override
    public String callCardTopApi(Order order) {
        return "SUCCESS";
    }


}
